# -------------------------------------
# [EZ-NET] CEzObject
# Base Object Class
#
# Date: 2001/04/05
# Version: 1.1.0
# Type: Class
#
# Copyright: (c) Easy Network
# Written by: Kumagai,Tomohiro
# Reference: http://www.ez-net.ne.jp/
# -------------------------------------

package CEzObject;


#### CONSTANTS ####
%SYS_INFO =
(
	'VersionInfo' => '1.1.0',
	'ClassInfo' => 'Easy Network Base Object'
);

%SYS_ENV =
(
	'informations' => '__informations__',	# Hash Key for Informations
	'attributes' => '__attributes__'	# Hash Key for Attributes
);

#### CONSTRUCTOR / DESTRUCTOR ####

# ---- Constructor ----
# $[r] own
# $[-] Owner Object

sub new()
{
	my ($that) = @_;
	
	my %implements =
	(
		$SYS_ENV{'attributes'} => {},
		$SYS_ENV{'informations'} => [\%SYS_INFO]
	);

	return bless {%implements}, $that;
}

# ---- Destructor ----
# $[r] unknown
# $[-] Owner Object

sub DESTROY()
{
	my ($this) = @_;
	
	$this->DeleteAttributes();
}


#### FUNCTIONS ####

# ---- Delete All Attributes ----
# $[r] unknown
# $[-] Owner Object

sub DeleteAttributes()
{
	my $this = shift;
	my ($attribute, $attributes) = (undef, $this->{$SYS_ENV{'attributes'}});
	
	foreach $attribute (keys %{$attributes})
	{
		undef $attributes->{$attribute};
	}
}

### METHODS ####

# ---- Adding Imprement ----
# $[r] unknown
# $[-] Owner Object
# $[1] Imprement name
# $[2] Default Value by Reference

sub AddImprement($$)
{
	my $this = shift;
	my ($name, $value) = @_;

	$this->{$name} = $value;
}

# ---- Adding Attribute ----
# $[r] unknown
# $[-] Owner Object
# $[1] Attribute name
# $[2] Default Value by Reference

sub AddAttribute($$)
{
	my $this = shift;
	my ($name, $value) = @_;

	return $this->SetAttribute($name, $value);
}

# ---- Get Attributes ----
# $[r] Referenced Value
# $[-] Owner Object
# $[1] Attribute Name

sub GetAttribute($)
{
	my $this = shift;
	my ($attribute) = @_;
	
	return $this->{$SYS_ENV{'attributes'}}->{$attribute};
}

# ---- Set Attributes ----
# $[r] unknown
# $[-] Owner Object
# $[1] Attribute Name
# $[2] Setting Value

sub SetAttribute($$)
{
	my $this = shift;
	my ($attribute, $value) = @_;
	
	return $this->{$SYS_ENV{'attributes'}}->{$attribute} = $value;
}

# --- Get Object Informations ---
# $[r] Array-Reference for Object Informations
# $[-] Owner Object

sub GetObjectInformations($)
{
	my $this = shift;
	return $this->{$SYS_ENV{'informations'}};
}

# --- Get Version Information ---
# $[r] version
# $[-] Owner Object

sub GetVersionInfo($)
{
	my $this = shift;
	my $info = $this->GetObjectInformations();

	return $info->[0]->{'VersionInfo'};
}

# --- Get Class Information ---
# $[r] Class Information
# $[-] Owner Object

sub GetClassInfo($)
{
	my $this = shift;
	my $info = $this->GetObjectInformations();

	return $info->[0]->{'ClassInfo'};
}

# --- Set Class Information ---
# $[r] Information Size
# $[-] Owner Object
# %[1] Set Informations

sub SetObjectInformation($$$)
{
	my $this = shift;
	my %info = {};

	$info{'VersionInfo'}	= shift;
	$info{'ClassInfo'}	= shift;

	return unshift @{$this->{$SYS_ENV{'informations'}}}, \%info;
}

1;
