# -------------------------------------
# [EZ-NET] CEzTime
# Easy Network Time Object
#
# Date: 2001/07/16
# Version: 1.0.1
# Type: Class
#
# Copyright: (c) Easy Network
# Written by: Kumagai,Tomohiro
# Reference: http://www.ez-net.ne.jp/
# -------------------------------------

package CEzTime;
use CEzObject;

@ISA = ('CEzObject');


# CONSTANTS
%SYS_INFO =
(
	'VersionInfo' => '1.0.1',
	'ClassInfo' => 'Easy Network Time Object'
);


sub new()
{
	my $that = shift;
	my $this = new CEzObject();


	$this->SetObjectInformation($SYS_INFO{'VersionInfo'}, $SYS_INFO{'ClassInfo'});

	$this->AddAttribute('datetime');
	$this->AddAttribute('format');
	$this->AddAttribute('week-name');
	$this->AddAttribute('month-name');

	bless $this, $that;


	$that->UseLocaltime();
	$that->SetFormat('<year.04>/<month.02>/<day.02> [<WEEK>] - <hour.02>:<minute.02>:<second.02>');

	$that->SetWeekName('SUN', 'MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT');
	$that->SetMonthName('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jly', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');

	return $that;
}

sub DESTROY()
{
	my $this = shift;
	$this->CEzObject::DESTROY();
}

# ---- ԏ LOCAL TIME ł킵܂
# $[-] Owner Object
# @[1] time

sub UseLocaltime($$)
{
	my $this = shift;
	my $time = $_[0] || time;

	$this->SetTime(localtime($time));
}

# ---- Zbg܂
# $[-] Owner Object
# $[1] 

sub SetFormat($$)
{
	my $this = shift;

	my($format) = @_;
	$this->SetAttribute('format', $format);
}

# ---- Ԃ𑮐ɃZbg܂
# $[-] Owner Object
# @[1] time

sub SetTime($@)
{
	my $this = shift;
	my(@data) = @_;

	my %buffer;

	$buffer{'year'}		= 1900 + $data[5];
	$buffer{'month'}	= $data[4] + 1;
	$buffer{'day'}		= $data[3];
	$buffer{'hour'}		= $data[2];
	$buffer{'minute'}	= $data[1];
	$buffer{'second'}	= $data[0];
	$buffer{'week'}		= $data[6];
	$buffer{'pass'}		= $data[7];

	$this->SetAttribute('datetime', {%buffer});
}

# ---- T̖Oݒ肵܂
# $[-] Owner Object
# @[1] List of Week Name

sub SetWeekName($@)
{
	my $this = shift;
	$this->SetAttribute('week-name', [@_]);
}

# ---- ̖Oݒ肵܂
# $[-] Owner Object
# @[1] List of Month Name

sub SetMonthName($@)
{
	my $this = shift;
	$this->SetAttribute('month-name', [@_]);
}

# w肳ꂽɊÂēt𐶐B
# $[r] t
# $[-] Owner Object

sub MakeDatetimeString($)
{
	my $this = shift;
	my($result) = $this->GetAttribute('format');

	my($tag);		# }b`^OLĂ
	my($temp, @temp);	# tp̍Ɨ̈

	my($left)	= '<';
	my($middle)	= '[^>]*';
	my($right)	= '>';

	my($lifecount) = 100;	# ItO



	my($datetime) = $this->GetAttribute('datetime');
	my($weekname) = $this->GetAttribute('week-name');
	my($monthname) = $this->GetAttribute('month-name');

	# tp^O < > ̎擾
	while ($result =~ /$left($middle)$right/)
	{
		$tag = $1;
		@temp = split(/\./, $tag);

		# -------------------------------
		if ($lifecount == 0)
		{
			$result .= "--ł؂܂B[$tag] --";
			last;
		}
		else
		{
			$lifecount--;
		}
		# -------------------------------

		if ($temp[0] eq 'WEEK')
		{
			# jւ̒u
			$result =~ s/$left$tag$right/$weekname->[$datetime->{'week'}]/;
		}
		elsif ($temp[0] eq 'MONTH')
		{
			# ւ̒u
			$result =~ s/$left$tag$right/$monthname->[$datetime->{'week'} - 1]/;
		}
		else
		{
			# Wln^O̕ϊ
			$temp = sprintf("\%$temp[1]d", $datetime->{$temp[0]});
			$result =~ s/$left$tag$right/$temp/;
		}
	}

	return $result;
}

1;
