# -------------------------------------
# [EZ-NET] ParamAnalyzer
# Analyze Parameter for CGI
#
# Date: 2001/07/16
# Version: 1.0.0
# Type: Class
#
# Copyright: (c) Easy Network
# Written by: Kumagai,Tomohiro
# Reference: http://www.ez-net.ne.jp/
# -------------------------------------

package ParamAnalyzer;
use CEzObject;

@ISA = ('CEzObject');


# CONSTANTS
%SYS_INFO =
(
	'VersionInfo' => '1.0.0',
	'ClassInfo' => 'Easy Network ParamAnalyzer Object'
);


# CONSTRUCTOR / DESTRUCTOR

sub new()
{
	my $that = shift;
	my $this = new CEzObject();

	my $method = $ENV{'REQUEST_METHOD'};

	$this->SetObjectInformation($SYS_INFO{'VersionInfo'}, $SYS_INFO{'ClassInfo'});

	$this->AddAttribute('params');

	$this->AddAttribute('post', ($method eq 'POST'));
	$this->AddAttribute('get', ($method eq 'GET'));
	$this->AddAttribute('others', '');

	return bless $this, $that;
}

sub DESTROY()
{
	my $this = shift;
	$this->CEzObject::DESTROY();
}

# METHOD

# --- Get ParamsString
# $[r] params string
# $[-] Owner Object

sub GetParamString
{
	my $params;
	my $this = shift;


	# ---- CUSTOM STRING ----
	$params = $this->GetAttribute('others');

	# ---- POST METHOD ----
	if ($this->GetAttribute('post'))
	{
		my $temp;

		read(STDIN, $temp, $ENV{'CONTENT_LENGTH'});
		$params .= "&$temp";
	}

	# ---- GET METHOD ----
	if ($this->GetAttribute('get'))
	{
		$params .= "&$ENV{'QUERY_STRING'}";
	}

	return $params;
}


# --- Encode for Param String
# $[r] Encoded String
# $[0] Plain Text

sub ParamEncode($)
{
	my($result) = @_;

	$result =~ s/([^a-zA-Z0-9_ ])/sprintf("%%%02X", unpack("C", $1))/eg;
	$result =~ s/ /\+/g;

	return $result;
}

# --- Encode for Param String
# $[r] Plain Text
# $[0] Encoded String

sub ParamDecode($)
{
	my($result) = @_;

	$result =~ s/\+/ /g;
	$result =~ s/%(..)/pack("c", hex($1))/ge;

	return $result;
}

# --- Analyze Parameters
# $[-] Owner Object

sub Analyze($)
{
	my $this = shift;

	my($param, %buffer);
	my(@params) = split(/&/, $this->GetParamString());

	foreach $param (@params)
	{
		if ($param)
		{
			my(@values, @work);
			@values = split(/=/, $param, 2);

			if (scalar(@values) == 1)
			{
				unshift (@values, 'notitle');
			}

			@work = @{$buffer{&ParamDecode($values[0])}};
			push (@work, &ParamDecode($values[1]));

			$buffer{&ParamDecode($values[0])} = [@work];
		}
	}

	$this->SetAttribute('params', {%buffer});
}

# --- Get Value with Array
# @[r] values
# $[-] Owner Object
# $[1] Param Name

sub GetValueArray($$)
{
	my $this = shift;

	my $name = $_[0]  || 'notitle';
	my $params = $this->GetAttribute('params');

	return @{$params->{$name}};
}

# --- Get Value with Scalar
# @[r] values
# $[-] Owner Object
# $[1] Param Name
# $[2] Array Index

sub GetValueScalar($$$)
{
	my $this = shift;
	my($name, $index) = @_;

	my @temp = $this->GetValueArray($name);
	return $temp[$index || 0];
}


# ---- Clear All Param
# $[-] Owner Object

sub ClearParams($)
{
	my $this = shift;
	$this->SetAttribute('params', undef);
}


# ---- Set Other Parameter String
# $[-] Owner Object
# $[1] Other Params

sub SetOtherParams($$)
{
	my $this = shift;
	my($params) = @_;

	$this->SetAttribute('others', $params);
}

1;
